@@ -1,17 +1,22 @@
-use crate::{Paint, Point, Scalar, Transform, Units};
+use crate::{Paint, Point, Scalar, SvgParserError, Transform, Units};
 use std::{
     fmt,
     ops::{Add, Mul},
     str::FromStr,
 };
 
+/// Common interface to all color representations
 pub trait Color {
+    /// Blend other color on top of this color
     fn blend_over(&self, other: &Self) -> Self;
 
+    /// Override alpha component of the color
     fn with_alpha(&self, alpha: Scalar) -> Self;
 
+    /// Convert color to sRGBA list
     fn to_rgba(&self) -> [u8; 4];
 
+    /// Convert color to sRGB list (alpha is discarded)
     fn to_rgb(&self) -> [u8; 3]
     where
         Self: Sized,
@@ -47,6 +52,7 @@ impl ColorU8 {
     }
 
     /*
+    ```
     fn blend_over(self, other: Self) -> Self {
         // Sa - source alpha
         // Sc - source color
@@ -68,6 +74,7 @@ impl ColorU8 {
         let _oc = lerp_u8x4(mul_u8x4(dc, da), sc, sa);
         todo!()
     }
+    ```
     */
 }
 
@@ -138,7 +145,18 @@ impl FromStr for ColorU8 {
     }
 }
 
-/// Alpha premultiplied RGBA color in the liniar color space (no gamma correction)
+impl fmt::Display for ColorU8 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let [r, g, b, a] = self.to_rgba();
+        write!(f, "#{:02x}{:02x}{:02x}", r, g, b)?;
+        if a != 255 {
+            write!(f, "{:02x}", a)?;
+        }
+        Ok(())
+    }
+}
+
+/// Alpha premultiplied RGBA color in the linear color space (no gamma correction)
 #[derive(Debug, Clone, Copy, PartialEq, Default)]
 pub struct LinColor([f32; 4]);
 
@@ -228,6 +246,10 @@ impl Paint for LinColor {
     fn transform(&self) -> Transform {
         Transform::identity()
     }
+
+    fn to_json(&self) -> Result<serde_json::Value, SvgParserError> {
+        Ok(serde_json::Value::String(self.to_string()))
+    }
 }
 
 impl Add<Self> for LinColor {
@@ -269,6 +291,12 @@ impl FromStr for LinColor {
     }
 }
 
+impl fmt::Display for LinColor {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        ColorU8::from(*self).fmt(f)
+    }
+}
+
 impl Color for Scalar {
     fn to_rgba(&self) -> [u8; 4] {
         let color = (linear_to_srgb(1.0 - (*self as f32)) * 255.0 + 0.5) as u8;
@@ -289,6 +317,17 @@ impl Color for Scalar {
 /// It was hard to optimize this function, even current version
 /// is slow because of the conditional jump. Lookup table is not working
 /// here as well it should be at least 4K in size an not cache friendly.
+///
+/// Precise implementation
+/// ```no_run
+/// pub fn linear_to_srgb(value: f32) -> f32 {
+///     if value <= 0.0031308 {
+///         value * 12.92
+///     } else {
+///         1.055 * value.powf(1.0 / 2.4) - 0.055
+///     }
+/// }
+/// ```
 #[inline]
 pub fn linear_to_srgb(x0: f32) -> f32 {
     if x0 <= 0.0031308 {
@@ -304,17 +343,6 @@ pub fn linear_to_srgb(x0: f32) -> f32 {
     }
 }
 
-/*
-#[inline]
-pub fn linear_to_srgb(value: f32) -> f32 {
-    if value <= 0.0031308 {
-        value * 12.92
-    } else {
-        1.055 * value.powf(1.0 / 2.4) - 0.055
-    }
-}
-*/
-
 #[inline]
 pub fn srgb_to_linear(value: f32) -> f32 {
     if value <= 0.04045 {
@@ -342,11 +370,12 @@ impl fmt::Display for ColorError {
 impl std::error::Error for ColorError {}
 
 /*
+```ignore
 const MASK_LOW: u32 = 0x00FF00FF;
 
 /// Calculate `[_, a1, _, a3] * b / 255`, where `a{0-3}` and `b` are `u8`
 ///
-/// Those optimisation comes from for these two formulas:
+/// Those optimization comes from for these two formulas:
 ///   1. `a + ar + ar^2 ... = a / (1 - r)` for all r in [0..1)
 ///   2. `t / 255 = (t / 256) / (1 - r)` where if `r = 1 / 256`
 ///
@@ -356,10 +385,10 @@ const MASK_LOW: u32 = 0x00FF00FF;
 /// Basically we get `v / 255 = ((v >> 8) + v) >> 8`
 ///
 /// This function also doing this operation at on two u8 at once by means of masking
-/// and then recomposing everyting in one value.
+/// and then recomposing everything in one value.
 ///
 /// References:
-///   - [Image Compasiting Fundamentals](https://www.cs.princeton.edu/courses/archive/fall00/cs426/papers/smith95a.pdf)
+///   - [Image Compositing Fundamentals](https://www.cs.princeton.edu/courses/archive/fall00/cs426/papers/smith95a.pdf)
 ///   - [Double blend trick](http://stereopsis.com/doubleblend.html)
 fn mul_u8x2(a: u32, b: u32) -> u32 {
     let m0 = (a & MASK_LOW) * b + 0x00800080;
@@ -389,13 +418,13 @@ pub fn lerp_u8x4(a: u32, b: u32, t: u32) -> u32 {
     let high = lerp_u8x2(a >> 8, b >> 8, t) << 8;
     low | high
 }
+```
 */
 
 #[cfg(test)]
 mod tests {
-    use crate::assert_approx_eq;
-
     use super::*;
+    use crate::assert_approx_eq;
 
     #[test]
     fn test_color_u8() {
@@ -435,6 +464,20 @@ mod tests {
             assert_approx_eq!(v, srgb_to_linear(linear_to_srgb(v)), 1e-4);
         }
     }
+
+    #[test]
+    fn test_display_parse() -> Result<(), ColorError> {
+        let c: ColorU8 = "#01020304".parse()?;
+        assert_eq!(c, ColorU8::new(1, 2, 3, 4));
+        assert_eq!(c.to_string(), "#01020304");
+
+        let c: ColorU8 = "#010203".parse()?;
+        assert_eq!(c, ColorU8::new(1, 2, 3, 255));
+        assert_eq!(c.to_string(), "#010203");
+
+        Ok(())
+    }
+
     /*
     #[test]
     fn test_mul_u8x4() {
