@@ -1,25 +1,29 @@
 use crate::{
     curve::line_offset, rasterize::Rasterizer, utils::clamp, BBox, Cubic, Curve, EllipArc,
-    ImageMut, LinColor, Line, Paint, Point, Quad, Scalar, Segment, Size, SvgPathParser,
-    SvgPathParserError, Transform, EPSILON,
+    ImageMut, LinColor, Line, Paint, Point, Quad, Scalar, Segment, Size, SvgParserError,
+    SvgPathParser, Transform, EPSILON,
 };
+use serde::{Deserialize, Serialize};
 use std::{
+    borrow::Cow,
     fmt,
     io::{Cursor, Read, Write},
     str::FromStr,
     usize,
 };
 
-/// Default flatness used during rasterizetion.
+/// Default flatness used during rasterization.
 /// Value of 0.05px gives good accuracy tradeoff.
 pub const DEFAULT_FLATNESS: Scalar = 0.05;
 
 /// The algorithm to use to determine the inside part of a shape, when filling it.
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
 pub enum FillRule {
     /// Fill area with non-zero winding number
+    #[serde(rename = "nonzero")]
     NonZero,
     /// Fill area with odd winding number
+    #[serde(rename = "evenodd")]
     EvenOdd,
 }
 
@@ -49,15 +53,18 @@ impl Default for FillRule {
 
 /// `LineJoin` defines the shape to be used at the corners of paths when they are stroked.
 /// See [SVG specification](https://www.w3.org/TR/SVG2/painting.html#LineJoin) for more details.
-#[derive(Debug, Clone, Copy, PartialEq, PartialOrd)]
+#[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Serialize, Deserialize)]
 pub enum LineJoin {
-    /// Continue path segments with lines untill they intersect. But only
+    /// Continue path segments with lines until they intersect. But only
     /// if `miter_length = stroke-width / sin(0.5 * eta)` is less than the miter argument.
+    #[serde(rename = "miter")]
     Miter(Scalar),
-    /// Connect path segments with straigh line.
+    /// Connect path segments with straight line.
+    #[serde(rename = "bevel")]
     Bevel,
     /// Round corner is to be used to join path segments.
     /// The corner is a circular sector centered on the join point.
+    #[serde(rename = "round")]
     Round,
 }
 
@@ -67,15 +74,18 @@ impl Default for LineJoin {
     }
 }
 
-/// `LineCap` specifies the shape to be used at the end of open subpaths when they are stroked.
+/// `LineCap` specifies the shape to be used at the end of open sub-paths when they are stroked.
 /// See [SVG specification](https://www.w3.org/TR/SVG2/painting.html#LineCaps) for more details.
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
 pub enum LineCap {
     /// Connect path segments with straight line.
+    #[serde(rename = "butt")]
     Butt,
     /// Add half-square to the end of the segments
+    #[serde(rename = "square")]
     Square,
     /// Add half-circle to the end of the segments
+    #[serde(rename = "round")]
     Round,
 }
 
@@ -86,17 +96,19 @@ impl Default for LineCap {
 }
 
 /// Style used to generate stroke
-#[derive(Debug, Clone, Copy, PartialEq, PartialOrd)]
+#[derive(Debug, Clone, Copy, PartialEq, PartialOrd, Serialize, Deserialize)]
 pub struct StrokeStyle {
     /// Width of the stroke
     pub width: Scalar,
     /// How to join offset segments
+    #[serde(default, skip_serializing_if = "crate::utils::is_default")]
     pub line_join: LineJoin,
     /// How to join segments at the ends of the path
+    #[serde(default, skip_serializing_if = "crate::utils::is_default")]
     pub line_cap: LineCap,
 }
 
-/// Non-empty collections of segments where end of each segments conisides with the start of the next one.
+/// Non-empty collections of segments where end of each segments coincides with the start of the next one.
 #[derive(Clone, PartialEq)]
 pub struct SubPath {
     /// List of segments representing SubPath
@@ -198,7 +210,7 @@ impl SubPath {
     }
 }
 
-/// Collection of the SubPath treated as a signle unit
+/// Collection of the SubPath treated as a single unit
 #[derive(Clone, PartialEq, Default)]
 pub struct Path {
     subpaths: Vec<SubPath>,
@@ -217,8 +229,14 @@ impl fmt::Debug for Path {
     }
 }
 
+impl fmt::Display for Path {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.to_svg_path())
+    }
+}
+
 impl Path {
-    /// Create path from the list of subpaths
+    /// Create path from the list of sub-paths
     pub fn new(subpaths: Vec<SubPath>) -> Self {
         Self { subpaths }
     }
@@ -263,7 +281,7 @@ impl Path {
     /// Stroke path
     ///
     /// Stroked path is the path constructed from original by offsetting by `distance/2` and
-    /// joinging it with the path offsetted by `-distance/2`.
+    /// joining it with the path offset by `-distance/2`.
     pub fn stroke(&self, style: StrokeStyle) -> Path {
         let mut subpaths = Vec::new();
         for subpath in self.subpaths.iter() {
@@ -321,7 +339,7 @@ impl Path {
             .fold(None, |bbox, subpath| Some(subpath.bbox(bbox, tr)))
     }
 
-    /// Calclulate size of the image required to render the path
+    /// Calculate size of the image required to render the path
     ///
     /// Returns:
     ///   - Size of the image
@@ -341,7 +359,7 @@ impl Path {
         Some((size, shift * tr, min))
     }
 
-    /// Rreverse order and direction of all segments
+    /// Reverse order and direction of all segments
     pub fn reverse(&self) -> Self {
         Self {
             subpaths: self.subpaths.iter().map(|s| s.reverse()).collect(),
@@ -390,7 +408,7 @@ impl Path {
     /// Save path in SVG path format.
     pub fn write_svg_path(&self, mut out: impl Write) -> std::io::Result<()> {
         for subpath in self.subpaths.iter() {
-            write!(&mut out, "M{:?} ", subpath.start())?;
+            write!(out, "M{:?} ", subpath.start())?;
             let mut segment_type: Option<u8> = None;
             for segment in subpath.segments().iter() {
                 match segment {
@@ -398,21 +416,21 @@ impl Path {
                         if segment_type.replace(b'L') != Some(b'L') {
                             out.write_all(b"L")?;
                         }
-                        write!(&mut out, "{:?} ", line.end())?;
+                        write!(out, "{:?} ", line.end())?;
                     }
                     Segment::Quad(quad) => {
                         let [_, p1, p2] = quad.points();
                         if segment_type.replace(b'Q') != Some(b'Q') {
                             out.write_all(b"Q")?;
                         }
-                        write!(&mut out, "{:?} {:?} ", p1, p2)?;
+                        write!(out, "{:?} {:?} ", p1, p2)?;
                     }
                     Segment::Cubic(cubic) => {
                         let [_, p1, p2, p3] = cubic.points();
                         if segment_type.replace(b'C') != Some(b'C') {
                             out.write_all(b"C")?;
                         }
-                        write!(&mut out, "{:?} {:?} {:?} ", p1, p2, p3)?;
+                        write!(out, "{:?} {:?} {:?} ", p1, p2, p3)?;
                     }
                 }
             }
@@ -606,7 +624,7 @@ impl PathBuilder {
     pub fn append_svg_path(
         &mut self,
         string: impl AsRef<[u8]>,
-    ) -> Result<&mut Self, SvgPathParserError> {
+    ) -> Result<&mut Self, SvgParserError> {
         for cmd in SvgPathParser::new(Cursor::new(string)) {
             cmd?.apply(self);
         }
@@ -659,7 +677,7 @@ impl PathBuilder {
         self.quad_to(p1, p2)
     }
 
-    /// Add cubic beizer curve
+    /// Add cubic bezier curve
     pub fn cubic_to(
         &mut self,
         p1: impl Into<Point>,
@@ -767,14 +785,14 @@ impl PathBuilder {
         self.close().move_to(init)
     }
 
-    /// Current possition of the builder
+    /// Current position of the builder
     pub fn position(&self) -> Point {
         self.position
     }
 }
 
 impl FromStr for Path {
-    type Err = SvgPathParserError;
+    type Err = SvgParserError;
 
     fn from_str(text: &str) -> Result<Path, Self::Err> {
         let mut builder = PathBuilder::new();
@@ -785,6 +803,26 @@ impl FromStr for Path {
     }
 }
 
+impl Serialize for Path {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        self.to_svg_path().serialize(serializer)
+    }
+}
+
+impl<'de> Deserialize<'de> for Path {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        Cow::<'de, str>::deserialize(deserializer)?
+            .parse()
+            .map_err(serde::de::Error::custom)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -814,7 +852,7 @@ mod tests {
     "#;
 
     #[test]
-    fn test_path_parse() -> Result<(), SvgPathParserError> {
+    fn test_path_parse() -> Result<(), SvgParserError> {
         // complicated path
         let path: Path = SQUIRREL.parse()?;
         let reference = Path::builder()
@@ -868,7 +906,7 @@ mod tests {
             .move_to((0.5, -3.0))
             .line_to((-11.0, -0.11))
             .build();
-        // unseparated scalars, implicit line segment
+        // not separated scalars, implicit line segment
         let p1: Path = "M.5-3-11-.11".parse()?;
         // other spaces, implicit relative line segment
         let p2: Path = " m.5,-3 -11.5\n2.89 ".parse()?;
@@ -889,7 +927,7 @@ mod tests {
     }
 
     #[test]
-    fn test_flatten() -> Result<(), SvgPathParserError> {
+    fn test_flatten() -> Result<(), SvgParserError> {
         let path: Path = SQUIRREL.parse()?;
         let tr = Transform::new_rotate(PI / 3.0).pre_translate(-10.0, -20.0);
         let lines: Vec<_> = path.flatten(tr, DEFAULT_FLATNESS, true).collect();
